PLOT_LINE = 'line'


# Deprecated
class MetricFigure:
    def __init__(self, title, y1_name, y1_type=PLOT_LINE, y2_name=None, y2_type=None):
        pass

    def push_metric(self, x, y1, y2=None):
        pass

